/*
 * PHP application for Asterisk
 * 
 * Copyright (C) 2005, David Eder
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <php_asterisk.h>
#include <../asterisk.h>
#include <asterisk/causes.h>
#include <asterisk/cdr.h>
#include <asterisk/channel.h>
#include <asterisk/cli.h>
#include <asterisk/file.h>
#include <asterisk/frame.h>
#include <asterisk/logger.h>
#include <asterisk/manager.h>
#include <asterisk/options.h>
#include <string.h>
#include <stdlib.h>

#define MAX_ARGS 128
#define PHP_CODE_SIZE 65536
static char *tdesc = "PHP for the dialplan";
static char *app = "PHP";
static char *synopsis = "Execute PHP code";
static char *descrip = "  PHP(filename[|option]):  Executes filename as PHP code.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

enum eval_ret_t { EVAL_RET_ERROR, EVAL_RET_OK, EVAL_RET_BAIL };
static int php_exec(struct ast_channel *chan, void *data)
{
        int res = 0;
        struct localuser *u;
        char *argv[MAX_ARGS];
        char buf[2048]="";
        char *tmp = (char *)buf;
        int argc = 0;
        char *stringp;
        char php_code[PHP_CODE_SIZE];

        if (!data || ast_strlen_zero(data)) {
                ast_log(LOG_WARNING, "PHP requires an argument (script)\n");
                return -1;
        }
        strncpy(buf, data, sizeof(buf) - 1);

        while ((stringp = strsep(&tmp, "|"))) {
                argv[argc++] = stringp;
        }
        argv[argc] = NULL;

	LOCAL_USER_ADD(u);

	php_asterisk_init(argc, argv PTSRMLS_CC);
	zend_first_try {

		snprintf(php_code, PHP_CODE_SIZE - 1, "define('CHANNEL', \"%s\");", chan->name);

		/* todo: fix this, it's the wrong way to do it! */

		/* asterisk.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONFIG_DIR', '%s');", AST_CONFIG_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_RUN_DIR', '%s');", AST_RUN_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOCKET', '%s');", AST_SOCKET);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_PID', '%s');", AST_PID);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_MODULE_DIR', '%s');", AST_MODULE_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SPOOL_DIR', '%s');", AST_SPOOL_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_VAR_DIR', '%s');", AST_VAR_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_LOG_DIR', '%s');", AST_LOG_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_AGI_DIR', '%s');", AST_AGI_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_KEY_DIR', '%s');", AST_KEY_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DB', '%s');", AST_DB);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_TMP_DIR', '%s');", AST_TMP_DIR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONFIG_FILE', '%s');", AST_CONFIG_FILE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOUNDS', '%s');", AST_SOUNDS);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_IMAGES', '%s');", AST_IMAGES);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONFIG_MAX_PATH', %d);", AST_CONFIG_MAX_PATH);

		/* causes.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_UNALLOCATED', %d);", AST_CAUSE_UNALLOCATED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NO_ROUTE_TRANSIT_NET', %d);", AST_CAUSE_NO_ROUTE_TRANSIT_NET);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NO_ROUTE_DESTINATION', %d);", AST_CAUSE_NO_ROUTE_DESTINATION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_CHANNEL_UNACCEPTABLE', %d);", AST_CAUSE_CHANNEL_UNACCEPTABLE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_CALL_AWARDED_DELIVERED', %d);", AST_CAUSE_CALL_AWARDED_DELIVERED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NORMAL_CLEARING', %d);", AST_CAUSE_NORMAL_CLEARING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_USER_BUSY', %d);", AST_CAUSE_USER_BUSY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NO_USER_RESPONSE', %d);", AST_CAUSE_NO_USER_RESPONSE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NO_ANSWER', %d);", AST_CAUSE_NO_ANSWER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_CALL_REJECTED', %d);", AST_CAUSE_CALL_REJECTED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NUMBER_CHANGED', %d);", AST_CAUSE_NUMBER_CHANGED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_DESTINATION_OUT_OF_ORDER', %d);", AST_CAUSE_DESTINATION_OUT_OF_ORDER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_INVALID_NUMBER_FORMAT', %d);", AST_CAUSE_INVALID_NUMBER_FORMAT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_FACILITY_REJECTED', %d);", AST_CAUSE_FACILITY_REJECTED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_RESPONSE_TO_STATUS_ENQUIRY', %d);", AST_CAUSE_RESPONSE_TO_STATUS_ENQUIRY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NORMAL_UNSPECIFIED', %d);", AST_CAUSE_NORMAL_UNSPECIFIED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NORMAL_CIRCUIT_CONGESTION', %d);", AST_CAUSE_NORMAL_CIRCUIT_CONGESTION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NETWORK_OUT_OF_ORDER', %d);", AST_CAUSE_NETWORK_OUT_OF_ORDER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NORMAL_TEMPORARY_FAILURE', %d);", AST_CAUSE_NORMAL_TEMPORARY_FAILURE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_SWITCH_CONGESTION', %d);", AST_CAUSE_SWITCH_CONGESTION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_ACCESS_INFO_DISCARDED', %d);", AST_CAUSE_ACCESS_INFO_DISCARDED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_REQUESTED_CHAN_UNAVAIL', %d);", AST_CAUSE_REQUESTED_CHAN_UNAVAIL);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_PRE_EMPTED', %d);", AST_CAUSE_PRE_EMPTED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_FACILITY_NOT_SUBSCRIBED', %d);", AST_CAUSE_FACILITY_NOT_SUBSCRIBED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_OUTGOING_CALL_BARRED', %d);", AST_CAUSE_OUTGOING_CALL_BARRED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_INCOMING_CALL_BARRED', %d);", AST_CAUSE_INCOMING_CALL_BARRED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_BEARERCAPABILITY_NOTAUTH', %d);", AST_CAUSE_BEARERCAPABILITY_NOTAUTH);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_BEARERCAPABILITY_NOTAVAIL', %d);", AST_CAUSE_BEARERCAPABILITY_NOTAVAIL);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_BEARERCAPABILITY_NOTIMPL', %d);", AST_CAUSE_BEARERCAPABILITY_NOTIMPL);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_CHAN_NOT_IMPLEMENTED', %d);", AST_CAUSE_CHAN_NOT_IMPLEMENTED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_FACILITY_NOT_IMPLEMENTED', %d);", AST_CAUSE_FACILITY_NOT_IMPLEMENTED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_INVALID_CALL_REFERENCE', %d);", AST_CAUSE_INVALID_CALL_REFERENCE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_INCOMPATIBLE_DESTINATION', %d);", AST_CAUSE_INCOMPATIBLE_DESTINATION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_INVALID_MSG_UNSPECIFIED', %d);", AST_CAUSE_INVALID_MSG_UNSPECIFIED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_MANDATORY_IE_MISSING', %d);", AST_CAUSE_MANDATORY_IE_MISSING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_MESSAGE_TYPE_NONEXIST', %d);", AST_CAUSE_MESSAGE_TYPE_NONEXIST);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_WRONG_MESSAGE', %d);", AST_CAUSE_WRONG_MESSAGE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_IE_NONEXIST', %d);", AST_CAUSE_IE_NONEXIST);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_INVALID_IE_CONTENTS', %d);", AST_CAUSE_INVALID_IE_CONTENTS);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_WRONG_CALL_STATE', %d);", AST_CAUSE_WRONG_CALL_STATE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_RECOVERY_ON_TIMER_EXPIRE', %d);", AST_CAUSE_RECOVERY_ON_TIMER_EXPIRE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_MANDATORY_IE_LENGTH_ERROR', %d);", AST_CAUSE_MANDATORY_IE_LENGTH_ERROR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_PROTOCOL_ERROR', %d);", AST_CAUSE_PROTOCOL_ERROR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_INTERWORKING', %d);", AST_CAUSE_INTERWORKING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_BUSY', %d);", AST_CAUSE_BUSY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_FAILURE', %d);", AST_CAUSE_FAILURE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NORMAL', %d);", AST_CAUSE_NORMAL);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NOANSWER', %d);", AST_CAUSE_NOANSWER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_CONGESTION', %d);", AST_CAUSE_CONGESTION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CAUSE_NOTDEFINED', %d);", AST_CAUSE_NOTDEFINED);

		/* cdr.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_FLAG_POSTED', %d);", AST_CDR_FLAG_POSTED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_FLAG_LOCKED', %d);", AST_CDR_FLAG_LOCKED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_FLAG_CHILD', %d);", AST_CDR_FLAG_CHILD);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_NOANSWER', %d);", AST_CDR_NOANSWER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_BUSY', %d);", AST_CDR_BUSY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_ANSWERED', %d);", AST_CDR_ANSWERED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_FAILED', %d);", AST_CDR_FAILED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_OMIT', %d);", AST_CDR_OMIT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_BILLING', %d);", AST_CDR_BILLING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_DOCUMENTATION', %d);", AST_CDR_DOCUMENTATION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_MAX_USER_FIELD', %d);", AST_MAX_USER_FIELD);

		/* channel.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_TRANSFER', %d);", AST_CDR_TRANSFER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_FORWARD', %d);", AST_CDR_FORWARD);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_CALLWAIT', %d);", AST_CDR_CALLWAIT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CDR_CONFERENCE', %d);", AST_CDR_CONFERENCE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_ADSI_UNKNOWN', %d);", AST_ADSI_UNKNOWN);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_ADSI_AVAILABLE', %d);", AST_ADSI_AVAILABLE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_ADSI_UNAVAILABLE', %d);", AST_ADSI_UNAVAILABLE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_ADSI_OFFHOOKONLY', %d);", AST_ADSI_OFFHOOKONLY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOFTHANGUP_DEV', %d);", AST_SOFTHANGUP_DEV);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOFTHANGUP_ASYNCGOTO', %d);", AST_SOFTHANGUP_ASYNCGOTO);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOFTHANGUP_SHUTDOWN', %d);", AST_SOFTHANGUP_SHUTDOWN);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOFTHANGUP_TIMEOUT', %d);", AST_SOFTHANGUP_TIMEOUT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOFTHANGUP_APPUNLOAD', %d);", AST_SOFTHANGUP_APPUNLOAD);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_SOFTHANGUP_EXPLICIT', %d);", AST_SOFTHANGUP_EXPLICIT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_DOWN', %d);", AST_STATE_DOWN);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_RESERVED', %d);", AST_STATE_RESERVED);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_OFFHOOK', %d);", AST_STATE_OFFHOOK);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_DIALING', %d);", AST_STATE_DIALING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_RING', %d);", AST_STATE_RING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_RINGING', %d);", AST_STATE_RINGING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_UP', %d);", AST_STATE_UP);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_BUSY', %d);", AST_STATE_BUSY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_DIALING_OFFHOOK', %d);", AST_STATE_DIALING_OFFHOOK);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_PRERING', %d);", AST_STATE_PRERING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_STATE_MUTE', %d);", AST_STATE_MUTE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DEVICE_UNKNOWN', %d);", AST_DEVICE_UNKNOWN);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DEVICE_NOT_INUSE', %d);", AST_DEVICE_NOT_INUSE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DEVICE_INUSE', %d);", AST_DEVICE_INUSE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DEVICE_BUSY', %d);", AST_DEVICE_BUSY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DEVICE_INVALID', %d);", AST_DEVICE_INVALID);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DEVICE_UNAVAILABLE', %d);", AST_DEVICE_UNAVAILABLE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_BRIDGE_DTMF_CHANNEL_0', %d);", AST_BRIDGE_DTMF_CHANNEL_0);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_BRIDGE_DTMF_CHANNEL_1', %d);", AST_BRIDGE_DTMF_CHANNEL_1);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_BRIDGE_REC_CHANNEL_0', %d);", AST_BRIDGE_REC_CHANNEL_0);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_BRIDGE_REC_CHANNEL_1', %d);", AST_BRIDGE_REC_CHANNEL_1);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_BRIDGE_IGNORE_SIGS', %d);", AST_BRIDGE_IGNORE_SIGS);

		/* cli.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_RESULT_SUCCESS', %d);", RESULT_SUCCESS);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_RESULT_SHOWUSAGE', %d);", RESULT_SHOWUSAGE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_RESULT_FAILURE', %d);", RESULT_FAILURE);

		/* file.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DIGIT_ANY', '%s');", AST_DIGIT_ANY);

		/* frame.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_G723_1', %d);", AST_FORMAT_G723_1);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_GSM', %d);", AST_FORMAT_GSM);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_ULAW', %d);", AST_FORMAT_ULAW);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_ALAW', %d);", AST_FORMAT_ALAW);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_G726', %d);", AST_FORMAT_G726);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_ADPCM', %d);", AST_FORMAT_ADPCM);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_SLINEAR', %d);", AST_FORMAT_SLINEAR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_LPC10', %d);", AST_FORMAT_LPC10);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_G729A', %d);", AST_FORMAT_G729A);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_SPEEX', %d);", AST_FORMAT_SPEEX);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_ILBC', %d);", AST_FORMAT_ILBC);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_MAX_AUDIO', %d);", AST_FORMAT_MAX_AUDIO);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_JPEG', %d);", AST_FORMAT_JPEG);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_PNG', %d);", AST_FORMAT_PNG);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_H261', %d);", AST_FORMAT_H261);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_H263', %d);", AST_FORMAT_H263);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_FORMAT_MAX_VIDEO', %d);", AST_FORMAT_MAX_VIDEO);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_HANGUP', %d);", AST_CONTROL_HANGUP);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_RING', %d);", AST_CONTROL_RING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_RINGING', %d);", AST_CONTROL_RINGING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_ANSWER', %d);", AST_CONTROL_ANSWER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_BUSY', %d);", AST_CONTROL_BUSY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_TAKEOFFHOOK', %d);", AST_CONTROL_TAKEOFFHOOK);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_OFFHOOK', %d);", AST_CONTROL_OFFHOOK);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_CONGESTION', %d);", AST_CONTROL_CONGESTION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_FLASH', %d);", AST_CONTROL_FLASH);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_WINK', %d);", AST_CONTROL_WINK);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_OPTION', %d);", AST_CONTROL_OPTION);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_PROGRESS', %d);", AST_CONTROL_PROGRESS);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_CONTROL_PROCEEDING', %d);", AST_CONTROL_PROCEEDING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_FLAG_REQUEST', %d);", AST_OPTION_FLAG_REQUEST);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_FLAG_ACCEPT', %d);", AST_OPTION_FLAG_ACCEPT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_FLAG_REJECT', %d);", AST_OPTION_FLAG_REJECT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_FLAG_QUERY', %d);", AST_OPTION_FLAG_QUERY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_FLAG_ANSWER', %d);", AST_OPTION_FLAG_ANSWER);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_FLAG_WTF', %d);", AST_OPTION_FLAG_WTF);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_TONE_VERIFY', %d);", AST_OPTION_TONE_VERIFY);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_TDD', %d);", AST_OPTION_TDD);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_RELAXDTMF', %d);", AST_OPTION_RELAXDTMF);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_OPTION_AUDIO_MODE', %d);", AST_OPTION_AUDIO_MODE);


		/* logger.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_LOG_DEBUG', %d);", __LOG_DEBUG);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_LOG_EVENT', %d);", __LOG_EVENT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_LOG_NOTICE', %d);", __LOG_NOTICE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_LOG_WARNING', %d);", __LOG_WARNING);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_LOG_ERROR', %d);", __LOG_ERROR);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_LOG_VERBOSE', %d);", __LOG_VERBOSE);

		/* manager.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_DEFAULT_MANAGER_PORT', %d);", DEFAULT_MANAGER_PORT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_EVENT_FLAG_SYSTEM', %d);", EVENT_FLAG_SYSTEM);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_EVENT_FLAG_CALL', %d);", EVENT_FLAG_CALL);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_EVENT_FLAG_LOG', %d);", EVENT_FLAG_LOG);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_EVENT_FLAG_VERBOSE', %d);", EVENT_FLAG_VERBOSE);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_EVENT_FLAG_COMMAND', %d);", EVENT_FLAG_COMMAND);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_EVENT_FLAG_AGENT', %d);", EVENT_FLAG_AGENT);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_EVENT_FLAG_USER', %d);", EVENT_FLAG_USER);

		/* options.h */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_VERBOSE_PREFIX_1', '%s');", VERBOSE_PREFIX_1);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_VERBOSE_PREFIX_2', '%s');", VERBOSE_PREFIX_2);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_VERBOSE_PREFIX_3', '%s');", VERBOSE_PREFIX_3);
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_VERBOSE_PREFIX_4', '%s');", VERBOSE_PREFIX_4);

		/* misc */
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_TDD_OFF', 0);");
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_TDD_ON', 1);");
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_TDD_TDD', 1);");
		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "define('AST_TDD_MATE', 2);");


		snprintf(php_code + strlen(php_code), PHP_CODE_SIZE - 1 - strlen(php_code), "require(\"%s\");", argv[0]);
		res = (zend_eval_string(php_code, NULL, "Asterisk" TSRMLS_CC)) ? EVAL_RET_OK : EVAL_RET_ERROR;
	} zend_catch {
		res = EVAL_RET_BAIL;
		/* int exit_status = EG(exit_status); */ \
	} zend_end_try();

	php_asterisk_shutdown(TSRMLS_C);

	LOCAL_USER_REMOVE(u);

	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, php_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
